require 'time'
require 'rack'
require 'sinatra/base'
require 'jungle_path/json'
require 'jungle_path/api/helpers'
require 'jungle_path/authentication/auth_provider'
require 'jungle_path/authentication/data_provider'
require 'jungle_path/authorization/filter'
require 'jungle_path/authorization/paths'
require 'jungle_path/query/filter'
require 'jungle_path/rack/basic_credentials'
require 'jungle_path/sql'

require_relative '../config/config' #for config settings use jungle.setting...
require_relative '../db/db'
require_relative '../schemas/schema'
require_relative '../auth/authorization'

module Server
  #class BaseAPI < Sinatra::Application
  class Base < Sinatra::Base
    configure do
      set :public_folder, jungle.application.public_dir
      set :root, jungle.application.root_dir
      set :dump_errors, true
      set :sessions, false
      set :logging, true
      mime_type :uql, 'text' #for sample queries.
      set :static, true
      puts "settings.root: #{settings.root}"
      puts "settings.public_folder: #{settings.public_folder}"
    end

    # Turns off "X-Frame-Options: SAMEORIGIN" http response header:
    #set :protection, :except => :frame_options

    extend JunglePath::API::Helpers::Defaults
    extend JunglePath::API::Helpers::StandardAPIs

		# Your own:
    extend Auth::Authorization

    def self.inherited(subclass)
      super
      subclass.instance_eval do
        set_default_rack_middleware jungle.application.logger, false
        set_default_error_handling jungle.application.logger

        helpers JunglePath::API::Helpers::DataCache
        helpers JunglePath::API::Helpers::Logging
        #set_logger jungle.application.logger
        helpers JunglePath::API::Helpers::Result
        helpers JunglePath::API::Helpers::Auth
        helpers Server::Base::Database

				# Your own:
        set_authorization jungle.route_access

        set_standard_api_get_log_file_list
        set_standard_api_get_log_file
        set_standard_api_query
      end
    end

    after do
      GC.start
    end

    before do
      puts ""
      puts "[request_start: #{Time.now.utc}]::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::"
      puts "[#{request.env['REMOTE_USER']}: #{request.request_method} #{request.url}"
      puts "(base_api - before do - authentication check)"
      puts "[params] #{params.to_h}" if jungle.debug.show_params
      if JunglePath::Authorization::Paths.is_open_path?(request, jungle.route_access)
        puts "is open path: #{request.path_info}"
      else
        puts "is secured path: #{request.path_info}"
        # These three request paths return user info, so force no_cache = true so that stale user data is not returned!
        no_cache = (request.path_info == '/authenticate' or request.path_info == '/current/user' or request.path_info == '/current/user/auth')
        puts "authenticate no_cache: #{no_cache}."
        #data_provider = JunglePath::Authentication::DataProvider::Test.new(Schema::Base.models)
        #def initialize db, user_model, models, roles, schema_filters, role_schema_filters, role_query_filters, restriction_query_filters, user_query_filters
        alternative_user_key_queries = [{name: :contact_id, query: "select id as alt_key from contact where user_id = ?"}]
        data_provider = JunglePath::Authentication::DataProvider::Default.new(self, cache, db, ::Schema::User, ::Schema::Base.models, jungle.roles, jungle.schema_filters, jungle.role_schema_filters, jungle.role_query_filters, jungle.restriction_query_filters, jungle.user_query_filters, jungle.role_table_filters, jungle.restriction_table_filters, jungle.user_table_filters, alternative_user_key_queries)
        auth_provider = JunglePath::Authentication::AuthProvider::Default.new
        authenticate auth_provider, data_provider, no_cache
      end
    end

    module Database
      def db
        Server::DB.instance
      end
    end
  end
end
