# can just require this:
#require 'jungle_path/schema'

# or these:
require 'jungle_path/schema/version'
require 'jungle_path/schema/base'
require 'jungle_path/schema/db'
require 'jungle_path/schema/auth'
require_relative 'views'

# application tables examples:
module Schema
	class Answer < Schema::Base
		self.description = ""
		define(
			[:id, :primary_key],
			[:question_id, :foreign_key, :question],
			[:description, :string],
			[:is_correct, :boolean],
			[:audit_user]
		)
	end

	class Contact < Schema::Base
		self.description = ""
		define(
			[:id, :primary_key],
			[:name, :string],
			[:email, :string],
			[:user_id, :foreign_key, :user, :unique],
			[:audit_user]
		)
	end

	class Test < Schema::Base
		self.description = ""
		define(
			[:id, :primary_key],
			[:name, :string],
			[:email, :string],
			[:user_id, :foreign_key, :user, :unique],
			[:audit_user]
		)
	end

	class FilterContact < Schema::Base
    self.description = "Test View filter stuff."
    define(
      [:id, :primary_key],
      [:name, :string],
      [:email, :string],
			[:user_id, :foreign_key, :user],
      [:audit_user]
    )
    self.view = Views.filter_contact
  end

	class FilterTest < Schema::Base
    self.description = "Test View filter stuff."
    define(
      [:id, :primary_key],
      [:name, :string],
      [:email, :string],
			[:user_id, :foreign_key, :user],
      [:audit_user]
    )
    self.view = Views.filter_test
  end

	class Practice < Schema::Base
		self.description = ""
		define(
			[:id, :primary_key],
			[:user_id, :foreign_key, :user],
			[:notes, :string],
			[:random_question_order, :boolean],
			[:random_answer_order, :boolean],
			[:audit_user]
		)
	end

	class PracticeQuiz < Schema::Base
		self.description = ""
		define(
			[:practice_id, :foreign_key, :practice, :primary_key],
			[:quiz_id, :foreign_key, :quiz, :primary_key],
			[:audit_user]
		)
		def self.plural_table_name
			"practice_quizzes"
		end
	end

	class PracticeQuestion < Schema::Base
		self.description = ""
		define(
			[:practice_id, :foreign_key, :practice, :primary_key],
			[:question_id, :foreign_key, :question, :primary_key],
			[:note, :string],
			[:audit_user]
		)
	end

	class PracticeAnswer < Schema::Base
		self.description = ""
		define(
			[:practice_id, :foreign_key, :practice, :primary_key],
			[:answer_id, :foreign_key, :answer, :primary_key],
			[:label, :string],
			[:selected, :boolean],
			[:audit_user]
		)
	end

	class Question < Schema::Base
		self.description = ""
		define(
			[:id, :primary_key],
			[:quiz_id, :foreign_key, :quiz],
			[:name, :string],
			[:description, :string],
			[:audit_user]
		)
	end

	class Quiz < Schema::Base
		self.description = ""
		define(
			[:id, :primary_key],
			[:name, :string],
			[:description, :string],
			[:private, :boolean, :default, true],
			[:audit_user]
		)
		def self.plural_table_name
			"quizzes"
		end
	end

	class Log < Schema::Base
		define(
			[:id, :primary_key],
			[:set_id, :integer],
			[:name, :string],
			[:type, :string],
			[:item, :string, :secure],
			[:timestamp, :timestamp]
		)
	end
end
