module JunglePath
	require 'jungle_path/gen/schema_tree'
	require 'jungle_path/gen/schema_tree/filter'
	require 'jungle_path/gen/schema_tree/node'
	module Authorization
		class Filter
			def initialize roles, schema_models_hash, role_permissions={}, role_restrictions={}, role_schema_filters={}, schema_filters={}
				@schema_models_hash = schema_models_hash
				@role_permissions = role_permissions
				@role_restrictions = role_restrictions
				eat_roles roles, role_permissions, role_restrictions, role_schema_filters, schema_filters
			end

			def is_root?
				@is_root = has_permission?(:root) if @is_root == nil
				@is_root
			end

			def is_user_admin?
				@is_user_admin = has_permission?(:auth_admin) if @is_user_admin == nil
				@is_user_admin
			end

			def has_access? permission_restriction_name
				has_permission?(permission_restriction_name) and !(has_restriction?(permission_restriction_name))
			end

			def has_permission? permission_name
				puts "@permissions.keys #{@permissions.keys}."
				puts "has_permission? #{permission_name}."
				symbol = permission_name.to_sym
				has_it = symbol == @permissions[symbol]
				has_it = :root == @permissions[:root] unless has_it
				has_it
			end

			def has_restriction? restriction_name
				symbol = restriction_name.to_sym
				symbol == @restrictions[symbol]
			end

			def permissions
				@permissions.keys
			end

			def restrictions
				@restrictions.keys
			end

			def schema_filter
				@schema_filter
			end

			def schema_node_tree
				@schema_node_tree = JunglePath::Gen::SchemaTree.gen_node_tree(@schema_models_hash, JunglePath::Gen::SchemaTree::Filter.new(@schema_filter)) unless @schema_node_tree
				@schema_node_tree
			end

			def roles
				@roles[0..-1]
			end

			def root_roles
				return @root_roles if @root_roles
				@root_roles = {}
				@role_permissions.each do |k, v|
					@root_roles[k] = k if v.include? :root
				end
				@root_roles
			end

			def to_s
				"roles: #{@roles}\npermissions: #{@permissions}\nrestrictions: #{@restrictions}\nschema_filter: #{@schema_filter}"
			end

			private

			def eat_roles roles, role_permissions, role_restrictions, role_schema_filters, schema_filters
				puts "eat_roles:"
				@roles = []
				@permissions = {}
				@restrictions = {}
				@schema_filter = nil
				if roles
					roles.each do |role|
						puts "role.class: #{role.class}."
						puts "role: #{role}."

						@roles << role[:name].to_sym
						permissions = role_permissions[role[:name].to_sym]
						if permissions
							permissions.each do |permission|
								puts "permission:  #{permission}."
								@permissions[permission.to_sym] = permission.to_sym
							end
						end
						restrictions = role_restrictions[role[:name].to_sym]
						if restrictions
							restrictions.each do |restriction|
								puts "restriction:  #{restriction}."
								@restrictions[restriction.to_sym] = restriction.to_sym
							end
						end
						@schema_filter = schema_filters[role_schema_filters[role[:name].to_sym]]
						puts "schema_filter:  #{@schema_filter}."
					end
				end
			end
		end
	end
end
