module JunglePath
	module Cache
		@hash = {}
		DEFAULT_TIMEOUT = 30 # seconds
		def self.[] key
			unwrap(@hash[key], key)
		end
		def self.[]= key, value
			@hash[key] = wrap(value)
		end
		def self.get key
			unwrap(@hash[key], key)
		end
		def self.set key, value, timeout=DEFAULT_TIMEOUT
			@hash[key] = wrap(value, timeout)
		end
		def self.clear key
			@hash.delete key
		end
		private
		def self.wrap value, timeout=DEFAULT_TIMEOUT
			{value: value, timestamp: Time.now.utc, timeout: timeout}
		end
		def self.unwrap item, key
			if item
				if Time.now.utc - item[:timestamp] > item[:timeout]
					@hash.delete key
					return nil
				end
				item[:timestamp] = Time.now.utc
				return item[:value]
			end
			item
		end
	end
end
