# config.rb
require 'singleton'

module JunglePath
  module Config
    # replacement for configatron gem.
    class Configuration
      attr_accessor :lock

      def initialize(parent=nil)
        @parent = parent
        @hash = {}
        @lock = false
      end

      def is_locked?
        return true if @lock
        return @parent.lock if @parent
        @lock
      end

      def method_missing(m, *args, &block)
        s = m.to_s
        return_value = nil
        if s.end_with? "="
          key = s[0..-2]
          if is_locked?
            raise "Configuration '#{key}' is locked."
          else
            @hash[key] = args[0]
          end
        else
          key = s
          if @hash.include?(key)
            return_value = @hash[key]
          else
            if is_locked?
              return_value = nil
            else
              return_value = Configuration.new self
              @hash[key] = return_value
            end
          end
        end
        return_value
      end
    end

    class ConfigurationRoot < Configuration
      include ::Singleton
    end

    def self.set_global_on
      # Monkey patch kernel module:
      ::Kernel.module_eval do
        #def configatron
        #  Config::ConfigurationRoot.instance
        #end
        def jungle
          Config::ConfigurationRoot.instance
        end
      end
    end
  end
end
