require 'sequel'

module JunglePath
	module DBAccess
		module IO
			require 'jungle_path/db_access/io/chunked_file_reader'
			require 'jungle_path/db_access/io/config'
			require 'jungle_path/db_access/io/copy'
			require 'jungle_path/db_access/io/db'
			require 'jungle_path/db_access/io/delete'
			require 'jungle_path/db_access/io/init_db'
			require 'jungle_path/db_access/io/insert'
			require 'jungle_path/db_access/io/schema'
			require 'jungle_path/db_access/io/select'
			require 'jungle_path/db_access/io/update'

			def self.connection_from_config(config)
				connection(
					database_type: config.type,
					user_name: config.user_name,
					database_name: config.name,
					host: config.host,
					extensions: config.extensions,
					password: config.password,
					port: config.port,
					options: config.options
				)
			end

			def self.connection_from_config_unknown_database(config)
				if config.type == 'postgres'
					connection_from_config_use_postgres_db(config)
				elsif config.type == 'tinytds'
					connection_from_config_use_ms_sql_server_db(config)
				else
					throw "Unknown database type: #{config.type}."
				end
			end

			def self.connection_from_config_use_ms_sql_server_db(config)
				db = connection(
					database_type: config.type,
					user_name: config.user_name,
					database_name: 'master',
					host: config.host,
					extensions: config.extensions,
					password: config.password,
					port: config.port,
					options: config.options
				)
			end

			def self.connection_from_config_use_postgres_db(config)
				db = connection(
					database_type: config.type,
					user_name: config.user_name,
					database_name: 'postgres',
					host: config.host,
					extensions: config.extensions,
					password: config.password,
					port: config.port,
					options: config.options
				)
			end

			def self.clone_config(config)
				JunglePath::DBAccess::IO::Config.new(
					name: config.name,
					type: config.type,
					user_name: config.user_name,
					password: config.password,
					host: config.host,
					extensions: config.extensions,
					port: config.port,
					options: config.options
				)
			end

			def self.connection(database_type:, user_name:, database_name:, host: "localhost", extensions: [], password: nil, port: nil, options: nil)
				Sequel.default_timezone = :utc
				colon_port = ":#{port}" if port
				options = {} unless options
				if password
					puts "Using db connection with explicit password!"
					connection_string = "#{database_type}://#{user_name}:#{password}@#{host}#{colon_port}/#{database_name}"
					puts "connection_string: #{connection_string}."
					db = Sequel.connect(connection_string, options)
					puts "connected."
				else
					puts "no explicit pw."
					connection_string = "#{database_type}://#{user_name}@#{host}#{colon_port}/#{database_name}"
					puts "connection_string: #{connection_string}."
					db = Sequel.connect(connection_string, options)
					puts "connected."
				end
				extensions.each do |extension|
					puts "Adding Sequel extension: #{extension}."
					db.extension extension
				end
				db
			end
		end
	end
end
