module JunglePath
	module DBAccess
		require 'jungle_path/db_access/meta/table'
		module IO
			require 'jungle_path/db_access/io'
			require 'jungle_path/db_access/io/select'
			require 'jungle_path/db_access/io/insert'
			require 'jungle_path/db_access/io/update'
			require 'jungle_path/db_access/io/delete'
			require 'jungle_path/db_access/io/schema'
			require 'jungle_path/db_access/io/copy'

			class DB
				attr_reader :select
				attr_reader :insert
				attr_reader :update
				attr_reader :delete
				attr_reader :schema
				attr_reader :copy
				attr_reader :base
				attr_reader :database_name
				attr_reader :config
				attr_reader :postgresql

				def initialize(config, logger=nil)
					@logger = logger
					@config = config
					@postgresql = config
					@db = JunglePath::DBAccess::IO.connection(
						database_type: config.type,
						user_name: config.user_name,
						database_name: config.name,
						host: config.host,
						extensions: config.extensions,
						password: config.password,
						port: config.port,
						options: config.options
					)
					@select = JunglePath::DBAccess::IO::Select.new @db, @logger
					@insert = JunglePath::DBAccess::IO::Insert.new @db, @logger
					@update = JunglePath::DBAccess::IO::Update.new @db, @logger
					@delete = JunglePath::DBAccess::IO::Delete.new @db, @logger
					@schema = JunglePath::DBAccess::IO::Schema.new @db, @logger
					@copy = JunglePath::DBAccess::IO::Copy.new @db, @logger
					@base = @db
					@database_name = config.name
				end

				def drop_table? table_name
					JunglePath::DBAccess::Meta::Table.drop? self, table_name
				end

				def table_exists? table_name
					JunglePath::DBAccess::Meta::Table.exists? self, table_name
				end

				def rename_table table_name, new_table_name
					JunglePath::DBAccess::Meta::Table.rename_table self, table_name, new_table_name
				end

				def create_table_like(from_table, to_table)
					JunglePath::DBAccess::Meta::Table.create_like(self, from_table, to_table)
				end

				def copy_table_data(from_table, to_table)
					JunglePath::DBAccess::Meta::Table.copy_data(self, from_table, to_table)
				end

				def reset_sequence_for_table(table_name)
					#max = @db[table_name.to_sym].max(:id) + 1
					#@db.run "alter sequence #{table_name}_id_seq restart with #{max}"
					@db.run "select setval('#{table_name}_id_seq', (select max(id)+1 from \"#{table_name}\"), false)"
				end

				def get_max_id_for_table(table_name, id_column_name=:id)
					ds = @db["select max(#{id_column_name}) as max_id from \"#{table_name}\""]
					result = ds.first
					max_id = result[:max_id]
				end

				def transaction
					@db.transaction do
						yield
					end
				end

				private

				def begin_transaction
					# not implemented :)
				end

				def commit_transaction
					# not implemented :)
				end

				def rollback_transaction
					# not implemented :)
				end
			end
		end
	end
end
