# gen_db.rb -- drops and creates a postgresql database from the commandline dropdb/createdb commands.
require 'jungle_path/db_access/io'

module JunglePath
  module DBAccess
    module Meta
      module DB
        def self.create(config)
          puts "JunglePath::DBAccess::Meta::DB.create: #{config.name}."
          db = JunglePath::DBAccess::IO.connection_from_config_use_postgres_db(config)
          sql = "create database #{config.name}"
          db.run sql
        end

        def self.drop!(config)
          puts "JunglePath::DBAccess::Meta::DB.drop: #{config.name}."
          kill_connections! config
          db = JunglePath::DBAccess::IO.connection_from_config_use_postgres_db(config)
          sql = "drop database #{config.name}"
          db.run sql
        end

        def self.drop?(config)
          if exists?(config)
            drop! config
          end
        end

        def self.reset!(config)
          drop? config
          create config
        end

        def self.exists?(config) # todo: fix to also work for ms sql server.
          puts "JunglePath::DBAccess::Meta::DB.exists? #{config.name}."
          exists = false
          db = JunglePath::DBAccess::IO.connection_from_config_unknown_database(config)
          sql = sql_query_db_existence(config)
          db.fetch(sql) do |row|
            exists = true
          end
          exists
        end

        def self.rename(config_from, to_name)
          puts "JunglePath::DBAccess::Meta::DB.rename: #{config_from.name} to #{to_name}."
          kill_connections! config_from
          db = JunglePath::DBAccess::IO.connection_from_config_use_postgres_db(config_from)
          sql = "alter database #{config_from.name} rename to #{to_name}"
          db.run sql
        end

        def self.rename?(config_from, to_name)
          if exists?(config_from)
            rename config_from, to_name
          end
        end

        def self.kill_connections!(config)
          db = JunglePath::DBAccess::IO.connection_from_config_use_postgres_db(config)
          sql = "select pg_terminate_backend(pid) from pg_stat_activity where datname = '#{config.name}'"
          db.run sql
        end

        private

        def self.sql_query_db_existence(config)
          if config.type == 'postgres'
            "select datname from pg_database where datname = '#{config.name}'"
          elsif config.type == 'tinytds'
            "select * from master.sys.databases where name = '#{config.name}'"
          else
            throw "Unknown database type: #{config.type}."
          end
        end
      end
    end
  end
end
