require 'jungle_path/schema'

module Schema
	class Key < Schema::Base
		define(
			[:id, :primary_key],
			[:key, :string, :unique, :secure, :not_null],
			[:user_id, :foreign_key, :user, :not_null, :index],
			[:expires_at, :timestamp, :not_null],
			[:audit_user]
		)
	end

	#class UserQueryFilter < Schema::Base
  #  self.description = "Links users to query_filters -- will be used to restrict certain users to only be able to query entities (such as products, etc.) as restricted by the related query_filter."
  #  define(
  #    [:user_id, :foreign_key, :user, :primary_key],
  #    [:query_filter_id, :foreign_key, :query_filter, :primary_key],
  #    [:audit_key]
  #  )
  #end

	#class QueryFilter < Schema::Base
  #  self.description = "misc. filters to apply against product/or other entities of the form: select product.id from product join table_x on table_x.product_id = product.id where table_x.some_column = 'abc'"
  #  define(
  #    [:id, :primary_key],
  #    [:name, :string, :desc, "Give this filter a name so you know who it should apply to."],
  #    [:base_table_name, :string, :desc, "Should be the name of the base table you are filtering on such as 'product'. Your filter will be applied to any queries containing tables which relate to the base table."],
  #    [:sub_select, :string, :desc, "Your sub_select query should select a set of ids from your base table. You can join to any other related tables and add where clause conditions. Do not use table aliases, instead use the full table names to avoid naming collisions as your sub_select will be hooked into queries generated by API users."],
	#		[:use_not_in, :boolean, :desc, "use a query like 'id not in (list....)' instead of 'id in (list...)'."],
  #    [:audit_key]
  #  )
  #end

	#class Role < Schema::Base # use roles to group privileges and restrictions
	#	define(
	#		[:id, :primary_key],
	#		[:name, :string, :unique_index, [:name], :not_null],
	#		[:description, :string],
	#		[:audit_key]
	#	)
	#end

	class User < Schema::Base
		self.description = "User of system."
		define(
			[:id, :primary_key],
			[:user_name, :string, :unique, :not_null, :alternate_key, :secure],
			[:hash, :string, :not_null, :secure, :calculated, :desc, "See password_hash.rb for details."],
			[:name, :string, :secure, :desc, "Your human name! :)"],
			[:first_name, :string, :secure],
			[:last_name, :string, :secure],
			[:email, :string, :secure],
			[:organization, :string, :secure],
			[:notes, :string, :desc, "For any misc. notes about this user including addition email addresses, phone numbers, etc."],
			[:active, :boolean, :default, true],
			[:is_valid, :boolean, :calculated],
			[:role, :string, :not_null],
			[:sms_verification_code, :string, :secure, :desc, "phone verification/sign-in code."],
			[:activation_key, :string, :secure],
			[:password_reset_code, :string, :secure],
			[:audit_user]
		)
	end

	class User
		@auth
		@query_filters
		@password
		attr_accessor :auth, :query_filters, :password
	end

	class Key
		def valid?
			#puts "expires_at: #{expires_at}."
			#puts "expires_at.class: #{expires_at.class}."
			expires_at > Time.now
		end
	end
end
